import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/auth/forget_password/bloc/forget_password_event.dart';
import 'package:syncrow_web/pages/auth/forget_password/bloc/forget_password_state.dart';
import 'package:syncrow_web/services/auth_api.dart';

class ForgetPasswordBloc
    extends Bloc<ForgetPasswordEvent, ForgetPasswordState> {
  ForgetPasswordBloc() : super(InitialForgetState()) {
    on<ChangePasswordEvent>(changePassword);
    on<StartTimerEvent>(_onStartTimer);
    on<StopTimerEvent>(_onStopTimer);
    on<UpdateTimerEvent>(_onUpdateTimer);
  }

  void _onUpdateTimer(UpdateTimerEvent event, Emitter<ForgetPasswordState> emit) {
    emit(TimerState(
        isButtonEnabled: event.isButtonEnabled,
        remainingTime: event.remainingTime));
  }

  final TextEditingController emailController = TextEditingController();
  final TextEditingController passwordController = TextEditingController();
  final TextEditingController otp = TextEditingController();
  final formKey = GlobalKey<FormState>();

  Timer? _timer;
  int _remainingTime = 0;

  Future<void> _onStartTimer(
      StartTimerEvent event, Emitter<ForgetPasswordState> emit) async {
    if (_validateInputs(emit)) return;
      print("StartTimerEvent received");
      if (_timer != null && _timer!.isActive) {
        print("Timer is already active");
        return;
      }
      _remainingTime = 60;
      add(UpdateTimerEvent(
          remainingTime: _remainingTime, isButtonEnabled: false));
      print("Timer started, initial remaining time: $_remainingTime");
      await AuthenticationAPI.sendOtp(email: emailController.text);
      _timer = Timer.periodic(const Duration(seconds: 1), (timer) {
        _remainingTime--;
        print("Timer tick, remaining time: $_remainingTime"); // Debug print
        if (_remainingTime <= 0) {
          _timer?.cancel();
          add(const UpdateTimerEvent(remainingTime: 0, isButtonEnabled: true));
          print("Timer finished"); // Debug print
        } else {
          add(UpdateTimerEvent(remainingTime: _remainingTime, isButtonEnabled: false));
        }
      });
  }

  void _onStopTimer(StopTimerEvent event, Emitter<ForgetPasswordState> emit) {
    _timer?.cancel();
    emit(TimerState(isButtonEnabled: true, remainingTime: 0));
  }

  Future<void> changePassword(
      ChangePasswordEvent event, Emitter<ForgetPasswordState> emit) async {
    try {
      emit(LoadingForgetState());
      await AuthenticationAPI.forgetPassword(
          password: passwordController.text, email: emailController.text);
      emit(SuccessForgetState());
    } catch (failure) {
      print(failure);
      emit(FailureForgetState(error: failure.toString()));
    }
  }

  String? validateEmail(String? value) {
    if (value == null || value.isEmpty) {
      return 'Email is required';
    } else if (!RegExp(r'^[^@]+@[^@]+\.[^@]+').hasMatch(value)) {
      return 'Enter a valid email address';
    }
    return null;
  }

  String? validateRegion(String? value) {
    if (value == null || value.isEmpty) {
      return 'Please select a region';
    }
    return null;
  }

  String? otpValidate(String? value) {
    if (value == null || value.isEmpty) {
      return 'Please select a region';
    }
    return null;
  }

  String? passwordValidator(String? value) {
    if (value == null || value.isEmpty) {
      return 'Please enter your password';
    }
    List<String> validationErrors = [];
    if (!RegExp(r'^(?=.*[a-z])').hasMatch(value)) {
      validationErrors.add(' - one lowercase letter');
    }
    if (!RegExp(r'^(?=.*[A-Z])').hasMatch(value)) {
      validationErrors.add(' - one uppercase letter');
    }
    if (!RegExp(r'^(?=.*\d)').hasMatch(value)) {
      validationErrors.add(' - one number');
    }
    if (!RegExp(r'^(?=.*[@$!%*?&])').hasMatch(value)) {
      validationErrors.add(' - one special character');
    }
    if (value.length < 8) {
      validationErrors.add(' - minimum 8 characters');
    }
    if (validationErrors.isNotEmpty) {
      return 'Password must contain at least:\n${validationErrors.join('\n')}';
    }
    return null;
  }

  final List<String> regions = [
    'North America',
    'South America',
    'Europe',
    'Asia',
    'Africa',
    'Australia',
    'Antarctica',
  ];


  bool _validateInputs(Emitter<ForgetPasswordState> emit) {
    emit(LoadingForgetState());
    final nameError = validateEmail(emailController.text);
    if (nameError != null) {
      emit(FailureForgetState(error:nameError ))  ;
      // CustomSnackBar.displaySnackBar(nameError);
      return true;
    }
    return false;
  }
}





//
//
// _login(LoginButtonPressed event, Emitter<LoginState> emit) async {
//   emit(LoginLoading());
//   if(isChecked) {
//     try {
//       if (event.username.isEmpty || event.password.isEmpty) {
//         CustomSnackBar.displaySnackBar('Please enter your credentials');
//         emit(const LoginFailure(error: 'Something went wrong'));
//         return;
//       }
//       token = await AuthenticationAPI.loginWithEmail(
//         model: LoginWithEmailModel(
//           email: event.username,
//           password: event.password,
//         ),
//       );
//     }
//     catch (failure) {
//       emit(const LoginFailure(error: 'Something went wrong'));
//       // emit(LoginFailure(error: failure.toString()));
//       return;
//     }
//     if (token.accessTokenIsNotEmpty) {
//       debugPrint('token: ${token.accessToken}');
//       FlutterSecureStorage storage = const FlutterSecureStorage();
//       await storage.write(
//           key: Token.loginAccessTokenKey, value: token.accessToken);
//
//       const FlutterSecureStorage().write(
//           key: UserModel.userUuidKey,
//           value: Token.decodeToken(token.accessToken)['uuid'].toString()
//       );
//       user = UserModel.fromToken(token);
//       emailController.clear();
//       passwordController.clear();
//       emit(LoginSuccess());
//     } else {
//       emit(const LoginFailure(error: 'Something went wrong'));
//     }
//   }
//   else{
//     emit(const LoginFailure(error: 'Accept terms and condition'));
//   }
// }
