import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:syncrow_web/pages/auth/forget_password/bloc/forget_password_bloc.dart';
import 'package:syncrow_web/pages/auth/forget_password/bloc/forget_password_event.dart';
import 'package:syncrow_web/pages/auth/forget_password/bloc/forget_password_state.dart';
import 'package:syncrow_web/pages/auth/login/view/login_page.dart';
import 'package:syncrow_web/pages/common/default_button.dart';
import 'package:syncrow_web/pages/common/first_layer.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/constants/assets.dart';
import 'package:syncrow_web/utils/style.dart';

class ForgetPasswordWebPage extends StatelessWidget {
  const ForgetPasswordWebPage({super.key});

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: BlocProvider(
        create: (context) => ForgetPasswordBloc(),
        child: BlocConsumer<ForgetPasswordBloc, ForgetPasswordState>(
          listener: (context, state) {
            if (state is LoadingForgetState) {
            } else if (state is FailureForgetState) {
              ScaffoldMessenger.of(context).showSnackBar(
                SnackBar(
                  content: Text(state.error),
                ),
              );
            }
          },
          builder: (context, state) {
            if (state is LoadingForgetState) {
              return const Center(child: CircularProgressIndicator());
            } else {
              return _buildForm(context, state);
            }
          },
        ),
      ),
    );
  }

  Widget _buildForm(BuildContext context, ForgetPasswordState state) {
    final forgetBloc = BlocProvider.of<ForgetPasswordBloc>(context);
    return FirstLayer(
      second: Container(
        margin: const EdgeInsets.all(50),
        decoration: BoxDecoration(
          color: Colors.black.withOpacity(0.3),
          borderRadius: const BorderRadius.all(Radius.circular(20)),
        ),
        child: Center(
          child: ListView(
            shrinkWrap: true,
            children: [
              Row(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  const Spacer(),
                  Expanded(
                    flex: 2,
                    child: SvgPicture.asset(
                      Assets.loginLogo,
                    ),
                  ),
                  const Spacer(),
                  Container(
                    decoration: BoxDecoration(
                      color: Colors.white.withOpacity(0.1),
                      borderRadius: const BorderRadius.all(Radius.circular(30)),
                      border: Border.all(color: ColorsManager.graysColor.withOpacity(0.2)),
                    ),
                    child: Form(
                      key: forgetBloc.formKey,
                      child: Padding(
                        padding: const EdgeInsets.symmetric(horizontal: 50, vertical: 20),
                        child: Column(
                          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            const SizedBox(height: 15),
                            const Text(
                              'Forget Password',
                              style: TextStyle(
                                  color: Colors.white,
                                  fontSize: 24,
                                  fontWeight: FontWeight.bold),
                            ),
                            Text(
                              'Please fill in your account information to retrieve your password',
                              style: smallTextStyle,
                            ),
                            const SizedBox(height: 30),
                            Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              mainAxisAlignment: MainAxisAlignment.start,
                              children: [
                                Text(
                                  "Country/Region",
                                  style: smallTextStyle,
                                ),
                                SizedBox(
                                  width: MediaQuery.of(context).size.width * 0.2,
                                  child: DropdownButtonFormField<String>(
                                    validator: forgetBloc.validateRegion,
                                    icon: const Icon(
                                      Icons.keyboard_arrow_down_outlined,
                                    ),
                                    decoration: textBoxDecoration()!.copyWith(
                                      hintText: null,
                                    ),
                                    hint: const Align(
                                      alignment: Alignment.centerLeft,
                                      child: Text(
                                        'Select your region/country',
                                        textAlign: TextAlign.center,
                                      ),
                                    ),
                                    isDense: true,
                                    style: const TextStyle(color: Colors.black),
                                    items: forgetBloc.regions.map((String region) {
                                      return DropdownMenuItem<String>(
                                        value: region,
                                        child: Text(region),
                                      );
                                    }).toList(),
                                    onChanged: (String? value) {
                                      print(value);
                                    },
                                  ),
                                )
                              ],
                            ),
                            const SizedBox(height: 20.0),
                            Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              mainAxisAlignment: MainAxisAlignment.start,
                              children: [
                                Text(
                                  "Account",
                                  style: smallTextStyle,
                                ),
                                SizedBox(
                                  width: MediaQuery.sizeOf(context).width * 0.2,
                                  child: TextFormField(
                                    validator: forgetBloc.validateEmail,
                                    controller: forgetBloc.emailController,
                                    decoration: textBoxDecoration()!.copyWith(hintText: 'Enter your email'),
                                    style: const TextStyle(color: Colors.black),
                                  ),
                                ),
                              ],
                            ),
                            const SizedBox(height: 20.0),
                            Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              mainAxisAlignment: MainAxisAlignment.start,
                              children: [
                                Text("One Time Password",
                                  style: smallTextStyle,),
                                SizedBox(
                                  width: MediaQuery.sizeOf(context).width * 0.2,
                                  child: TextFormField(
                                    validator: forgetBloc.otpValidate,
                                    keyboardType: TextInputType.visiblePassword,
                                    controller: forgetBloc.otp,
                                    decoration: textBoxDecoration()!.copyWith(
                                      hintText: 'Enter Code',
                                      suffixIcon: SizedBox(
                                        width: 100,
                                        child: Center(
                                          child: InkWell(
                                            onTap:  () {
                                              BlocProvider.of<ForgetPasswordBloc>(context).add(StartTimerEvent());
                                            },
                                            child: Text(
                                              'Get Code ${state is TimerState && !state.isButtonEnabled ? "(${state.remainingTime.toString()})" : ""}',
                                              style: TextStyle(
                                                color: state is TimerState && !state.isButtonEnabled
                                                    ? Colors.grey
                                                    : ColorsManager.btnColor,
                                              ),
                                            ),
                                          ),
                                        ),
                                      ),
                                    ),
                                    style: const TextStyle(color: Colors.black),
                                  ),
                                ),
                              ],
                            ),
                            const SizedBox(height: 20.0),
                            Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              mainAxisAlignment: MainAxisAlignment.start,
                              children: [
                                Text("Password",
                                  style: smallTextStyle,),
                                SizedBox(
                                  width: MediaQuery.sizeOf(context).width * 0.2,
                                  child: TextFormField(
                                    validator: forgetBloc.passwordValidator,
                                    keyboardType: TextInputType.visiblePassword,
                                    controller: forgetBloc.passwordController,
                                    decoration: textBoxDecoration()!.copyWith(
                                      hintText: 'At least 8 characters',
                                    ),
                                    style: const TextStyle(color: Colors.black),
                                  ),
                                ),
                              ],
                            ),
                            const SizedBox(
                              height: 10,
                            ),
                            const SizedBox(height: 20.0),
                            SizedBox(
                              width: MediaQuery.sizeOf(context).width * 0.2,
                              child: DefaultButton(
                                backgroundColor: ColorsManager.btnColor,
                                child: const Text('Submit'),
                                onPressed: () {
                                  if (forgetBloc.formKey.currentState!.validate()) {
                                    forgetBloc.add(ChangePasswordEvent());
                                  }
                                },
                              ),
                            ),
                            SizedBox(height: 10,),
                            SizedBox(
                              width: MediaQuery.sizeOf(context).width * 0.2,
                              child: Row(
                                mainAxisAlignment: MainAxisAlignment.center,
                                crossAxisAlignment: CrossAxisAlignment.center,
                                children: [
                                  const Flexible(
                                      child: Text(
                                        "Do you have an account? ",
                                        style: TextStyle(color: Colors.white),
                                      )),
                                  InkWell(
                                    onTap: () {
                                      Navigator.pop(context);
                                    },
                                    child: const Flexible(
                                        child: Text(
                                          "Sign in",
                                        )),
                                  ),
                                ],
                              ),
                            )
                          ],
                        ),
                      ),
                    ),
                  ),
                  const Spacer(),
                ],
              ),
            ],
          ),
        ),
      ),
    );
  }
}
